/*
 * Copyright (C) 2025 The Phosh Developers
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#pragma once

#include <glib.h>

G_BEGIN_DECLS

const char * gm_mcc_to_iso (const char *mcc_str, GError **err);

G_END_DECLS
